'use strict';

exports.__esModule = true;

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _lodash = require('lodash');

var _testRun = require('../errors/test-run');

var _getCallsite = require('../errors/get-callsite');

var _getCallsite2 = _interopRequireDefault(_getCallsite);

var _commands = require('../test-run/commands');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TestController = function () {
    function TestController(testRun) {
        (0, _classCallCheck3.default)(this, TestController);

        this.testRun = testRun;
        this.executionChain = _pinkie2.default.resolve();
        this.apiMethods = this._createAPIMethodsList();
        this.callsiteWithoutAwait = null;
    }

    TestController.prototype._createAPIMethodsList = function _createAPIMethodsList() {
        return (0, _keys2.default)(TestController.prototype).filter(function (name) {
            return !/^_/.test(name) && typeof TestController.prototype[name] === 'function';
        });
    };

    // NOTE: we track missing `awaits` by exposing a special custom Promise to user code.
    // Action or assertion is awaited if:
    // a)someone used `await` so Promise's `then` function executed
    // b)Promise chained by using one of the mixed-in controller methods
    //
    // In both scenarios, we check that callsite that produced Promise is equal to the one
    // that is currently missing await. This is required to workaround scenarios like this:
    //
    // var t2 = t.click('#btn1'); // <-- stores new callsiteWithoutAwait
    // await t2;                  // <-- callsiteWithoutAwait = null
    // t.click('#btn2');          // <-- stores new callsiteWithoutAwait
    // await t2.click('#btn3');   // <-- without check it will set callsiteWithoutAwait = null, so we will lost tracking


    TestController.prototype._createExtendedPromise = function _createExtendedPromise(promise, callsite) {
        var _this = this;

        var extendedPromise = promise.then(_lodash.identity);
        var originalThen = extendedPromise.then;

        var ensureAwait = function ensureAwait() {
            if (_this.callsiteWithoutAwait === callsite) _this.callsiteWithoutAwait = null;
        };

        extendedPromise.then = function () {
            ensureAwait();
            return originalThen.apply(this, arguments);
        };

        this.apiMethods.forEach(function (name) {
            var controller = _this;

            extendedPromise[name] = function () {
                ensureAwait();
                return controller['_' + name + 'Impl'].apply(controller, arguments);
            };
        });

        return extendedPromise;
    };

    TestController.prototype._enqueueAction = function _enqueueAction(apiMethodName, CmdCtor, cmdArgs) {
        var _this2 = this;

        this._checkForMissingAwait();

        var callsite = (0, _getCallsite2.default)(apiMethodName);
        var command = null;

        try {
            command = new CmdCtor(cmdArgs);
        } catch (err) {
            err.callsite = callsite;
            throw err;
        }

        this.executionChain = this.executionChain.then(function () {
            return _this2.testRun.executeCommand(command, callsite);
        });
        this.callsiteWithoutAwait = callsite;

        return this._createExtendedPromise(this.executionChain, callsite);
    };

    TestController.prototype._checkForMissingAwait = function _checkForMissingAwait() {
        if (this.callsiteWithoutAwait) throw new _testRun.MissingAwaitError(this.callsiteWithoutAwait);
    };

    // API implementation
    // We need implementation methods to obtain correct callsites. If we use plain API
    // methods in chained wrappers then we will have callsite for the wrapped method
    // in this file instead of chained method callsite in user code.


    TestController.prototype._clickImpl = function _clickImpl(selector, options) {
        return this._enqueueAction('click', _commands.ClickCommand, { selector: selector, options: options });
    };

    TestController.prototype._rightClickImpl = function _rightClickImpl(selector, options) {
        return this._enqueueAction('rightClick', _commands.RightClickCommand, { selector: selector, options: options });
    };

    TestController.prototype._doubleClickImpl = function _doubleClickImpl(selector, options) {
        return this._enqueueAction('doubleClick', _commands.DoubleClickCommand, { selector: selector, options: options });
    };

    TestController.prototype._hoverImpl = function _hoverImpl(selector, options) {
        return this._enqueueAction('hover', _commands.HoverCommand, { selector: selector, options: options });
    };

    TestController.prototype._dragImpl = function _dragImpl(selector, dragOffsetX, dragOffsetY, options) {
        return this._enqueueAction('drag', _commands.DragCommand, { selector: selector, dragOffsetX: dragOffsetX, dragOffsetY: dragOffsetY, options: options });
    };

    TestController.prototype._dragToElementImpl = function _dragToElementImpl(selector, destinationSelector, options) {
        return this._enqueueAction('dragToElement', _commands.DragToElementCommand, { selector: selector, destinationSelector: destinationSelector, options: options });
    };

    // API


    TestController.prototype.click = function click(selector, options) {
        return this._clickImpl(selector, options);
    };

    TestController.prototype.rightClick = function rightClick(selector, options) {
        return this._rightClickImpl(selector, options);
    };

    TestController.prototype.doubleClick = function doubleClick(selector, options) {
        return this._doubleClickImpl(selector, options);
    };

    TestController.prototype.hover = function hover(selector, options) {
        return this._hoverImpl(selector, options);
    };

    TestController.prototype.drag = function drag(selector, dragOffsetX, dragOffsetY, options) {
        return this._dragImpl(selector, dragOffsetX, dragOffsetY, options);
    };

    TestController.prototype.dragToElement = function dragToElement(selector, destinationSelector, options) {
        return this._dragToElementImpl(selector, destinationSelector, options);
    };

    return TestController;
}();

exports.default = TestController;
module.exports = exports['default'];